@@ -254,10 +254,12 @@ class TarExperimentPlotter(ExperimentPlotter[LT], Generic[LT]):
         self._plot_stop_criteria()
         self._set_axes(x_lim, y_lim)
         plt.legend(bbox_to_anchor=(1.05, 1), loc="upper left")
-
+        plt.tight_layout()
         if filename is not None:
             plt.savefig(filename, bbox_inches="tight")
-            plt.close()
+        else:
+            plt.show()
+        plt.close()
 
     def user_show(
         self,
@@ -275,7 +277,9 @@ class TarExperimentPlotter(ExperimentPlotter[LT], Generic[LT]):
         plt.legend(bbox_to_anchor=(1.05, 1), loc="upper left")
         if filename is not None:
             plt.savefig(filename, bbox_inches="tight")
-            plt.close()
+        else:
+            plt.show()
+        plt.close()
 
     def wss_at_target(self, target: float) -> float:
         for it in self.it_axis:
